@@ -22,6 +22,7 @@
 #endif /* HAVE_CONFIG_H */
 
 #include <stdio.h>
+#include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -187,6 +188,10 @@ wmf_error_t wmf_lite_create (wmfAPI** API_return,unsigned long flags,wmfAPI_Opti
 
 	API->color_data = 0;
 
+	API->store.attrlist = 0;
+	API->store.count = 0;
+	API->store.max = 0;
+
 /* Library error state:
  */
 	API->err = wmf_E_None;
