@@ -35,6 +35,45 @@
 
 #include "font.h"
 
+#ifdef _WIN32
+
+extern char* _libwmf_get_fontdir();
+
+static char* _libwmf_get_xtra_fontmap (void)
+{
+	static char retval[1000] = "";
+
+	if (retval[0] == '\0')
+	{	strcpy (retval, _libwmf_get_fontdir ());
+		strcat (retval, "\\share\\libwmf\\fonts\\fontmap");
+	}
+
+	return retval;
+}
+
+#undef WMF_XTRA_FONTMAP
+#define WMF_XTRA_FONTMAP _libwmf_get_xtra_fontmap ()
+
+static char* remap_font_file_name(wmfAPI* API,char* filename)
+{
+	/* If filename is prefixed with the compile-time WMF_FONTDIR,
+	 * substitute the run-time font directory.
+	 */
+	char* retval;
+	if (strncmp (filename, WMF_FONTDIR "/", strlen (WMF_FONTDIR "/")) == 0)
+	{	retval = wmf_malloc (API,strlen (filename) - strlen (WMF_FONTDIR) + strlen (_libwmf_get_fontdir ()) + 1);
+		strcpy (retval,_libwmf_get_fontdir ());
+		strcat (retval,filename + strlen (WMF_FONTDIR));
+	}
+	else
+	{	retval = wmf_strdup (API,filename);
+	}
+
+	return retval;
+}
+
+#endif
+
 static void ipa_font_add_wmf (wmfAPI* API,wmfFontMap* mapping)
 {	wmfFontmapData* font_data = (wmfFontmapData*) ((wmfFontData*) API->font_data)->user_data;
 
@@ -508,6 +547,11 @@ float wmf_ipa_font_stringwidth (wmfAPI* API,wmfFont* font,char* str)
 	int i;
 	int length;
 
+	if ((face == 0) || (str == 0))
+	{	WMF_DEBUG (API,"wmf_ipa_font_stringwidth: NULL face or str - ??");
+		return 0;
+	}
+
 	FT_Set_Char_Size (face,0,12 * 64,300,100);
 
 	FT_Set_Transform (face,0,0);
@@ -521,7 +565,9 @@ float wmf_ipa_font_stringwidth (wmfAPI* API,wmfFont* font,char* str)
 	length = strlen (str);
 
 	for (i = 0; i < length; i++)
-	{	/* convert character code to glyph index */ /* === Should this be unsigned ?? ==== */
+	{	/* convert character code to glyph index */
+		/* ==== Should this be unsigned ??  ==== */
+
 		glyph_index = FT_Get_Char_Index (face,str[i]);
 
 		if (use_kerning && previous && glyph_index)
@@ -530,10 +576,12 @@ float wmf_ipa_font_stringwidth (wmfAPI* API,wmfFont* font,char* str)
 			width += (float) (delta.x >> 6);
 		}
 
-		/* load glyph image into the slot.  DO NOT RENDER IT! */
-		FT_Load_Glyph (face,glyph_index,FT_LOAD_DEFAULT);
+		if (glyph_index)
+		{	/* load glyph image into the slot.  DO NOT RENDER IT! */
+			FT_Load_Glyph (face,glyph_index,FT_LOAD_DEFAULT);
 
-		width += (float) (face->glyph->advance.x >> 6);
+			if (face->glyph) width += (float) (face->glyph->advance.x >> 6);
+		}
 
 		previous = glyph_index;
 	}
@@ -1280,7 +1328,12 @@ void wmf_ipa_font_map (wmfAPI* API,wmfFont* font)
  */
 	mapping = ipa_font_std (API,font);
 
-	if (mapping) ipa_font_face (API,font,mapping);
+	if (mapping)
+	{	if (ipa_font_face (API,font,mapping)) return;
+	}
+
+	WMF_ERROR (API,"wmf_ipa_font_map: failed to load *any* font!");
+	API->err = wmf_E_BadFile;
 }
 
 /**
@@ -1324,12 +1377,14 @@ static wmfXML_FontInfo* ipa_font_sys_map (wmfAPI* API,wmfFont* font)
  * Checks XML Fontmap entry is valid before loading face.
  */
 static FT_Face ipa_font_sys_face (wmfAPI* API,wmfFont* font,wmfXML_FontInfo* FI)
-{	wmfFontmapData* font_data = (wmfFontmapData*) ((wmfFontData*) API->font_data)->user_data;
+{	
+#if 0
+	wmfFontmapData* font_data = (wmfFontmapData*) ((wmfFontData*) API->font_data)->user_data;
+	struct stat stat_buf;
+#endif
 
 	FT_Face face = 0;
 
-	struct stat stat_buf;
-
 	if (FI == 0) return (0);
 
 	if (FI->name == 0)
@@ -1434,8 +1489,6 @@ static FT_Face ipa_font_gs_face (wmfAPI* API,wmfFont* font,wmfGS_FontInfo* FI)
 
 	FT_Face face = 0;
 
-	unsigned int i;
-
 	char* aalias = 0;
 
 	if (FI == 0) return (0);
@@ -1604,10 +1657,16 @@ static FT_Face ipa_font_face_open (wmfAPI* API,char* ps_name,char* glyphs,char*
 
 	struct stat stat_buf;
 
+#ifdef _WIN32
+	glyphs = remap_font_file_name (API,glyphs);
+#endif
 	if (stat (glyphs,&stat_buf))
 	{	WMF_ERROR (API,"unable to stat font file:");
 		WMF_ERROR (API,glyphs);
 		API->err = wmf_E_BadFile;
+#ifdef _WIN32
+		wmf_free (API,glyphs);
+#endif
 		return (0);
 	}
 
@@ -1615,9 +1674,15 @@ static FT_Face ipa_font_face_open (wmfAPI* API,char* ps_name,char* glyphs,char*
 	{	WMF_ERROR (API,"Failed to open font:");
 		WMF_ERROR (API,glyphs);
 		API->err = wmf_E_DeviceError;
+#ifdef _WIN32
+		wmf_free (API,glyphs);
+#endif
 		return (0);
 	}
 
+#ifdef _WIN32
+	metrics = remap_font_file_name (API,metrics);
+#endif
 	if (metrics)
 	{	if (stat (metrics,&stat_buf))
 		{	WMF_DEBUG (API,"unable to stat font metrics file:");
