@@ -20,7 +20,8 @@
 /* This is called by wmf_play() the *first* time the meta file is played
  */
 static void wmf_fig_device_open (wmfAPI* API)
-{	wmf_fig_t* ddata = WMF_FIG_GetData (API);
+{	
+/*	wmf_fig_t* ddata = WMF_FIG_GetData (API); */
 
 	WMF_DEBUG (API,"~~~~~~~~wmf_[fig_]device_open");
 
@@ -30,7 +31,8 @@ static void wmf_fig_device_open (wmfAPI* API)
 /* This is called by wmf_api_destroy()
  */
 static void wmf_fig_device_close (wmfAPI* API)
-{	wmf_fig_t* ddata = WMF_FIG_GetData (API);
+{
+/*	wmf_fig_t* ddata = WMF_FIG_GetData (API); */
 
 	WMF_DEBUG (API,"~~~~~~~~wmf_[fig_]device_close");
 
@@ -40,7 +42,8 @@ static void wmf_fig_device_close (wmfAPI* API)
 /* This is called from the beginning of each play for initial page setup
  */
 static void wmf_fig_device_begin (wmfAPI* API)
-{	wmf_fig_t* ddata = WMF_FIG_GetData (API);
+{	
+	wmf_fig_t* ddata = WMF_FIG_GetData (API);
 
 	wmfStream* out = ddata->out;
 
@@ -53,24 +56,8 @@ static void wmf_fig_device_begin (wmfAPI* API)
 	float ratio_def;
 	float ratio_page;
 
-	float Ox;
-	float Oy;
-
-	float xScale;
-	float yScale;
-	
-	int i;
-	
-	int zero;
-	int colno;
-
-	char clr[10];
-	char c;
-
 	time_t t;
 
-	float fu;
-	
 	WMF_DEBUG (API,"~~~~~~~~wmf_[fig_]device_begin");
 
 	if (out == 0) return;
